import org.webbitserver.HttpHandler
import com.cattura.share.utils.io.ProcessHelper
import org.webbitserver.HttpControl
import org.webbitserver.HttpRequest
import org.webbitserver.HttpResponse

class LogDownloaderHttpHandler implements HttpHandler
{
    private static final String URI_PREFIX = "/logs/"
    private LogDownloaderPlugin _plugin
    
    private File _storageDir;
    
    public LogDownloaderHttpHandler (final LogDownloaderPlugin $plugin, final File $storageDir)
    {
        _plugin = $plugin;
        _storageDir = $storageDir;
    }
    
    /*
     * HTTP interface.
     *
     * Can either accept two timestamps separated as {from}/{to} or 
     * a capture session path.
     */
    public void handleHttpRequest (final HttpRequest $request, final HttpResponse $response, final HttpControl $control)
    {
        if ($request.uri().startsWith(URI_PREFIX))
        {
            final String path = $request.uri().substring(URI_PREFIX.length())
            final File captureSessionPath = new File(_storageDir, path)
            if (captureSessionPath.exists())
            {
                handleHttpRequestGivenPath($response, captureSessionPath)
            }
            else
            {
                def (from, to) = path.split("/").collect() {
                    return it.isLong() ? new Date(it as Long) : null
                }
                
                handleHttpRequestGivenTimestamps($response, from, to)
            }
            
            return
        }
        
        $control.nextHandler()
    }
    
    /*
     * /logs/{captureRequestPath}
     */
    private void handleHttpRequestGivenPath (final HttpResponse $response, final File $path)
    {
        final File zip = _plugin.createCaptureSessionLogArchive($path)
        zip.withInputStream() {
            $response.content(it.bytes)
        }
        zip.delete()
        $response.header("Content-Disposition","attachment; filename=${$path.name}-logs.zip")
                 .header("Content-Type","application/zip")
                 .end()
    }
    
    /*
     * /logs/{fromTimestamp}/{toTimestamp}
     */
    private void handleHttpRequestGivenTimestamps (final HttpResponse $response, Date $from, Date $to)
    {
        if (!$to)
        {
            $to = new Date()
        }
        
        if (!$from)
        {
            $response.status(500).content("Invalid 'from' date specified.").end()
            return
        }

        final File zip = _plugin.createLogArchive(from: $from, to: $to)
        zip.withInputStream() {
            $response.content(it.bytes)
        }
        zip.delete()
        $response.header("Content-Disposition","attachment; filename=${getLogFilename()}")
                 .header("Content-Type","application/zip")
                 .end()
    }
    
    private String getLogFilename ()
    {
        final String unitName = new ProcessHelper("uname").arg("-n").call()
        final String dateTime = new Date().toString()
        return "catturalogs_${unitName}_${dateTime}.zip".replace(" ", "_").replace(":", "-")
    }
}